<?php

namespace Drupal\tome;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Site\Settings;

/**
 * Provides methods for reading and writing the index file.
 */
trait ContentIndexerTrait {

  /**
   * Writes content to the index.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   An entity to be indexed.
   *
   * @throws \Exception
   *   Throws an exception when the index file cannot be written to.
   */
  protected function indexContent(ContentEntityInterface $entity) {
    $dependencies = [];
    foreach ($entity as $field) {
      if ($field instanceof EntityReferenceFieldItemListInterface) {
        foreach ($field->referencedEntities() as $referenced_entity) {
          if ($referenced_entity instanceof ContentEntityInterface) {
            $dependencies[] = TomeHelper::getContentName($referenced_entity);
          }
        }
      }
    }
    if (!$entity->isDefaultTranslation()) {
      $dependencies[] = TomeHelper::getContentNameFromParts($entity->getEntityTypeId(), $entity->uuid());
    }
    $destination = $this->getContentIndexFilePath();
    $directory = dirname($destination);
    file_prepare_directory($directory, FILE_CREATE_DIRECTORY);
    $handle = fopen($destination, 'c+');
    $lock = flock($handle, LOCK_EX);
    if (!$lock) {
      throw new \Exception('Unable to acquire lock for the index file.');
    }
    $contents = stream_get_contents($handle);
    if (empty($contents)) {
      $index = [];
    }
    else {
      $index = json_decode($contents, TRUE);
    }
    $dependencies = array_values(array_unique($dependencies));
    $index[TomeHelper::getContentName($entity)] = $dependencies;
    ftruncate($handle, 0);
    rewind($handle);
    fwrite($handle, json_encode($index, JSON_PRETTY_PRINT));

    flock($handle, LOCK_UN);
  }

  /**
   * Removes content from the index.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   An entity to be indexed.
   *
   * @throws \Exception
   *   Throws an exception when the index file cannot be written to.
   */
  protected function unIndexContent(ContentEntityInterface $entity) {
    $destination = $this->getContentIndexFilePath();
    $handle = fopen($destination, 'c+');
    $lock = flock($handle, LOCK_EX);
    if (!$lock) {
      throw new \Exception('Unable to acquire lock for the index file.');
    }
    $contents = stream_get_contents($handle);
    if (empty($contents)) {
      return;
    }
    $index = json_decode($contents, TRUE);
    $name = TomeHelper::getContentName($entity);
    if (isset($index[$name])) {
      unset($index[$name]);
    }
    foreach ($index as &$dependencies) {
      $dependencies = array_diff($dependencies, [$name]);
    }
    ftruncate($handle, 0);
    rewind($handle);
    fwrite($handle, json_encode($index, JSON_PRETTY_PRINT));

    flock($handle, LOCK_UN);
  }

  /**
   * Gets the contents of the index.
   *
   * @return bool|array
   *   The index, or FALSE if there was an error.
   */
  protected function getContentIndex() {
    $destination = $this->getContentIndexFilePath();
    if (!file_exists($destination)) {
      return FALSE;
    }
    $contents = file_get_contents($destination);
    return json_decode($contents, TRUE);
  }

  /**
   * Deletes the index file.
   */
  protected function deleteContentIndex() {
    $destination = $this->getContentIndexFilePath();
    if (is_file($destination)) {
      unlink($destination);
    }
  }

  /**
   * Gets the index file path.
   *
   * @return string
   *   The index file path.
   */
  protected function getContentIndexFilePath() {
    return Settings::get('tome_content_directory', '../content') . '/meta/index.json';
  }

}
